package pack;
import pack.Learning;
import pack.QTools;
import pack.ColorSensor;
import lejos.hardware.Brick;
import lejos.hardware.BrickFinder;
import lejos.hardware.Button;
//import lejos.hardware.Sound;
//import lejos.hardware.lcd.LCD;
import lejos.hardware.motor.Motor;
import lejos.hardware.port.Port;
import lejos.hardware.sensor.EV3ColorSensor;
import lejos.hardware.sensor.SensorMode;
import lejos.robotics.RegulatedMotor;
import lejos.robotics.navigation.MovePilot;
import lejos.utility.Delay;

public class LineTrace {

	public static byte NUMBER_ACTIONS = 11;
	public static byte NUMBER_STATES = 4;

	static MovePilot pilot;
	static RegulatedMotor leftMotor = Motor.B; 
	static RegulatedMotor rightMotor = Motor.C; 
	static ColorSensor color1, color2;
	static int[] states;
	
	byte[] env = new byte[2];
	byte motorCommand = 0;
	
	static QTools q = new QTools(NUMBER_ACTIONS, NUMBER_STATES);
	static Learning Q = new Learning(NUMBER_ACTIONS, NUMBER_STATES, q);

	//@SuppressWarnings("deprecation")
	
	public static void main(String[] args) {
		Brick brick1 = BrickFinder.getDefault();
		Port s1 = (Port) brick1.getPort("S1");
		Port s2 = (Port) brick1.getPort("S4");
		EV3ColorSensor sensor1 = new EV3ColorSensor(s1);
		EV3ColorSensor sensor2 = new EV3ColorSensor(s2);
		sensor1.getColorIDMode();
		sensor2.getColorIDMode();
		color1 = new ColorSensor(sensor1);
		color2 = new ColorSensor(sensor2);
		pilot = new MovePilot(6, 13, leftMotor, rightMotor);
		states = new int[2];
	
		while(Button.ESCAPE.isUp()){
			
			states = getEnvironment(states, color1, color2);
			
			int a = Q.getAction(states);
			
			int command = q.getCommand(a);
			
			motorCommand(pilot, command);
			Delay.msDelay(500);
			
		} exit(sensor1, sensor2);

	}	

	/* Void instruction for closing the program
	 * Safely with the sensor specifically used.
	 */
	public static void exit(EV3ColorSensor s1, EV3ColorSensor s2){ 
		pilot.stop();
		s1.close();
		s2.close();
		System.exit(0);
	}

	/* 基本的には弱く右、弱く左、強く右、強く左、
	 * 前、止まるの6通りの動きをする。このプログ
	 * ラムではより自然な動きを実現するために、後
	 * ろへ下がる動作を入れてある。下がる動作では
	 * 報酬が著しく低いため学習後は消える可能性が
	 * 高い。
	 */
	public static void motorCommand(MovePilot p, int command){
		switch(command){
		case 3:  // Right turn (soft)
			//p.rotate(-15); 
			Motor.B.setSpeed(160);
			Motor.C.setSpeed(80);
			Motor.B.forward();
			Motor.C.forward();
			break;
			
		case 4:  // Right turn (hard)
			//p.rotate(-45); 
			Motor.B.setSpeed(200);
			Motor.C.setSpeed(40);
			Motor.B.forward();
			Motor.C.forward();
			break;
			
		case 2:  // Left turn (hard)
			//p.rotate(45);
			Motor.B.setSpeed(40);
			Motor.C.setSpeed(200);
			Motor.B.forward();
			Motor.C.forward();
			break;
			
		case 1:  // Left turn (soft)
			//p.rotate(15);
			Motor.B.setSpeed(80);
			Motor.C.setSpeed(160);
			Motor.B.forward();
			Motor.C.forward();
			break;

		case 5:  // Forward
			//p.travel(5);
			Motor.B.setSpeed(120);
			Motor.C.setSpeed(120);
			Motor.B.forward();
			Motor.C.forward();
			break;

		case 6:  // backward
			//p.travel(5);
			Motor.B.setSpeed(120);
			Motor.C.setSpeed(120);
			Motor.B.backward();
			Motor.C.backward();
			break;
		case 7:  // Left turn back(soft)
			//p.travel(5);
			Motor.B.setSpeed(80);
			Motor.C.setSpeed(160);
			Motor.B.backward();
			Motor.C.backward();
			break;
		case 8:  // Left turn back(hard)
			//p.travel(5);
			Motor.B.setSpeed(40);
			Motor.C.setSpeed(200);
			Motor.B.backward();
			Motor.C.backward();
			break;
		case 9:  // Right turn back(soft)
			//p.travel(5);
			Motor.B.setSpeed(160);
			Motor.C.setSpeed(80);
			Motor.B.backward();
			Motor.C.backward();
			break;
		case 10:  // Forward
			//p.travel(5);
			Motor.B.setSpeed(200);
			Motor.C.setSpeed(40);
			Motor.B.backward();
			Motor.C.backward();
			break;

			
		case 0:  // Stop
			p.stop();
			break;
			
		default: 
			break;
		}	
	}

	/* 黒以外は白と同一視
	 */
	public static int[] getEnvironment(int[] states2, ColorSensor color1, ColorSensor color2){
		int sensor1 = color1.pressed();
		int sensor2 = color2.pressed();
		states2 = new int[2];
		if (sensor1 == 7) {
			sensor1 = 1;
		}else{
			sensor1 = 0;
		}
		if (sensor2 == 7) {
			sensor2 = 1;
		}else{
			sensor2 = 0;
		}
		states2[0] = sensor1;
		states2[1] = sensor2;
		return states2;
	}
}